async def gather(hub, profiles):
    """
    load k8s profiles from credential files

    Example:
    .. code-block:: yaml

        k8s:
          profile_name:
            kube_config_path: '~/.kube/config'
            context: kubernetes-admin
    """
    sub_profiles = {}
    for profile, ctx in profiles.get("k8s", {}).items():
        sub_profiles[profile] = ctx
    return sub_profiles



from typing import Dict


def create_waiter_config(
    hub,
    default_delay: int = 15,
    default_max_attempts: int = 40,
    timeout_config: Dict = None,
) -> Dict[str, int]:
    """
    Create a waiter configuration that can be used as input in waiter calls. The configuration is based on
    the default values and the customized value in timeout_config.

    Args:
        hub: The redistributed pop central hub.
        default_delay: The amount of time in seconds to wait between attempts.
        default_max_attempts: The maximum number of attempts to be made.
        timeout_config: Customized timeout configuration containing delay and max attempts.

    Returns:
        {"delay": delay-time, "max_attempts": max-attempts}
    """
    result = {"delay": default_delay, "max_attempts": default_max_attempts}
    if timeout_config:
        if "delay" in timeout_config:
            result["delay"] = timeout_config["delay"]
        if "max_attempts" in timeout_config:
            result["max_attempts"] = timeout_config["max_attempts"]
    return result


def create_waiter(hub, **kwargs):
    class CustomWaiter:
        def __init__(self):
            self.api_class_name = kwargs.get("api_class_name")
            self.operation = kwargs.get("operation")
            self.arguments = kwargs.get("arguments")
            self.result_arguments = kwargs.get("result_arguments")
            self.acceptor_function = kwargs.get("acceptor_function")
            self.waiter_config = kwargs.get("waiter_config")

    return CustomWaiter()



from typing import Any
from typing import Dict


def convert_raw_storage_class_to_present(hub, storage_class) -> Dict[str, Any]:
    if not storage_class:
        return None

    result = {"resource_id": storage_class.metadata.name}
    skip_attributes = [
        "api_version",
        "kind",
        "metadata.annotations",
        "metadata.creation_timestamp",
        "metadata.generation",
        "metadata.managed_fields",
        "metadata.resource_version",
        "metadata.uid",
        "metadata.self_link",
    ]
    result.update(
        hub.tool.k8s.marshaller.marshal(
            k8s_object=storage_class,
            skip_attributes=skip_attributes,
            skip_empty_values=True,
        )
    )

    if result.get("allow_volume_expansion") is None:
        result["allow_volume_expansion"] = False

    return result



from typing import Any
from typing import Dict


def convert_raw_cluster_role_to_present(hub, cluster_role) -> Dict[str, Any]:
    if not cluster_role:
        return None

    result = {"resource_id": cluster_role.metadata.name}
    skip_attributes = [
        "api_version",
        "kind",
        "metadata.creation_timestamp",
        "metadata.managed_fields",
        "metadata.resource_version",
        "metadata.uid",
    ]
    result.update(
        hub.tool.k8s.marshaller.marshal(
            k8s_object=cluster_role,
            skip_attributes=skip_attributes,
            skip_empty_values=True,
            include_empty_string_keys=["api_groups"],
        )
    )
    return result



def parse(hub, spec: str, resource_name: str):
    k8s_resource_definitions = spec.get("definitions")
    k8s_resources = parse_paths(spec.get("paths"))

    if resource_name not in k8s_resources:
        hub.log.error(f"unsupported resource {resource_name} for kubernetes provider")
        raise NameError(resource_name)

    parsed_data = {}
    for action in ("create", "delete", "update", "get", "list"):
        resource_metadata = identify_resource_metadata(
            k8s_resources[resource_name], action
        )
        parsed_data[action] = parse_resource_metadata(
            hub, action, k8s_resource_definitions, resource_metadata
        )
    return parsed_data


def parse_paths(paths):
    resource_list = {}
    for uri, apis in paths.items():
        for http_method, details in apis.items():
            if "x-kubernetes-group-version-kind" not in details:
                continue

            # resource category : group/version/kind
            group_version_kind = [
                details["x-kubernetes-group-version-kind"].get("group")
                if details["x-kubernetes-group-version-kind"].get("group")
                else "core",
                details["x-kubernetes-group-version-kind"].get("version"),
                details["x-kubernetes-group-version-kind"].get("kind"),
            ]
            resource_category = "/".join([item for item in group_version_kind if item])
            api_class = (
                "".join(
                    item[0].upper() + item[1:]
                    for item in details["tags"].pop().split("_")
                )
                + "Api"
            )
            spec_data = {
                "description": details["description"],
                "http_method": http_method,
                "operation_id": details["operationId"],
                "method_params": details["parameters"]
                if "parameters" in details
                else None,
                "responses": details["responses"] if "responses" in details else None,
                "api_uri": uri,
                "api_params": apis["parameters"] if "parameters" in apis else None,
                "api_class": api_class,
            }
            if resource_category in resource_list:
                resource_list[resource_category].append(spec_data)
            else:
                resource_list[resource_category] = [
                    spec_data,
                ]
    return resource_list


def identify_resource_metadata(spec_data_list, action):
    spec_data_mapping = {
        "create": {
            "operation_prefix": "create",
            "http_method": "post",
            "exclusion_terms": [],
        },
        "update": {
            "operation_prefix": "replace",
            "http_method": "put",
            "exclusion_terms": [],
        },
        "delete": {
            "operation_prefix": "delete",
            "http_method": "delete",
            "exclusion_terms": ["Collection"],
        },
        "get": {
            "operation_prefix": "read",
            "http_method": "get",
            "exclusion_terms": [],
        },
        "list": {
            "operation_prefix": "list",
            "http_method": "get",
            "exclusion_terms": ["Namespaced"],
        },
    }

    if not spec_data_list:
        return {}

    filtered = [
        spec_data
        for spec_data in spec_data_list
        if (
            spec_data["http_method"] == spec_data_mapping[action]["http_method"]
            and spec_data["operation_id"].startswith(
                spec_data_mapping[action]["operation_prefix"]
            )
            and not any(
                map(
                    spec_data["operation_id"].__contains__,
                    spec_data_mapping[action]["exclusion_terms"],
                )
            )
        )
    ]
    return filtered[0]


def parse_resource_metadata(hub, action, resource_definitions, resource_metadata):
    payload_type, params = parse_spec_parameters(
        hub,
        action,
        resource_definitions,
        resource_metadata.get("method_params"),
    )
    return {
        "doc": resource_metadata.get("description"),
        "api_class": resource_metadata.get("api_class"),
        "function_name": hub.tool.format.case.snake(
            resource_metadata.get("operation_id")
        ),
        "params": params,
        "payload_type": None
        if not payload_type
        else "".join([item.capitalize() for item in payload_type.split(".") if item]),
    }


def parse_spec_parameters(hub, action, resource_definitions, method_params):
    payload_type = None
    parameters = {}
    if not method_params:
        return payload_type, parameters

    excluded_parameters = ["apiVersion", "kind"]
    if action == "create":
        props = {}
        for param in method_params:
            if "in" not in param or param["in"] != "body":
                continue
            schema = param.get("schema")
            definition_name = schema.get("$ref").split("/").pop()
            definition = resource_definitions.get(definition_name)
            payload_type = definition_name
            props = definition.get("properties")
            break
        for param_name, details in props.items():
            if param_name in excluded_parameters:
                continue
            parameters[param_name] = {
                "default": None,
                "doc": "\n            ".join(
                    hub.tool.format.wrap.wrap(details.get("description"), width=96)
                ),
                "param_type": hub.pop_create.k8s.type.translate_type(
                    details.get("type") if "$ref" not in details else "map"
                ),
                "required": True,
                "target": "kwargs",  # arg, kwargs, hardcoded
                "target_type": "mapping",
            }
    if action == "delete":
        parameters["metadata"] = {
            "default": None,
            "doc": "\n            ".join(
                hub.tool.format.wrap.wrap(
                    "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.",
                    width=96,
                )
            ),
            "param_type": "Dict",
            "required": True,
            "target": "kwargs",
            "target_type": "mapping",
        }

    return payload_type, parameters



from typing import Any
from typing import Dict


def convert_raw_secret_to_present(hub, secret) -> Dict[str, Any]:
    if not secret:
        return None

    result = {"resource_id": secret.metadata.name}
    skip_attributes = [
        "api_version",
        "kind",
        "metadata.creation_timestamp",
        "metadata.managed_fields",
        "metadata.resource_version",
        "metadata.uid",
    ]
    result.update(
        hub.tool.k8s.marshaller.marshal(
            k8s_object=secret,
            skip_attributes=skip_attributes,
            skip_empty_values=True,
        )
    )

    if result.get("immutable") is None:
        result["immutable"] = False

    return result



from typing import Any
from typing import Dict


def convert_raw_service_account_to_present(hub, service_account) -> Dict[str, Any]:
    if not service_account:
        return None

    result = {"resource_id": service_account.metadata.name}
    skip_attributes = [
        "api_version",
        "kind",
        "metadata.creation_timestamp",
        "metadata.managed_fields",
        "metadata.resource_version",
        "metadata.uid",
    ]
    result.update(
        hub.tool.k8s.marshaller.marshal(
            k8s_object=service_account,
            skip_attributes=skip_attributes,
            skip_empty_values=True,
        )
    )

    if not result.get("automount_service_account_token"):
        result["automount_service_account_token"] = False

    return result



# https://pop.readthedocs.io/en/latest/tutorial/quickstart.html#adding-configuration-data


CONFIG = {}

SUBCOMMANDS = {
    "k8s": {
        "help": "Create idem_k8s state modules by parsing openapi spec",
        "dyne": "pop_create",
    },
}

CLI_CONFIG = {
    "resource": {
        "subcommands": ["k8s"],
        "dyne": "pop_create",
    },
}

DYNE = {
    "acct": ["acct"],
    "exec": ["exec"],
    "states": ["states"],
    "tool": ["tool"],
    "pop_create": ["autogen"],
}



import pathlib
import urllib.request

from dict_tools.data import NamespaceDict

HAS_LIBS = (True,)


def __virtual__(hub):
    return HAS_LIBS


def context(hub, ctx, directory: pathlib.Path):
    ctx = hub.pop_create.idem_cloud.init.context(ctx, directory)
    ctx.servers = [None]
    ctx.service_name = "k8s"
    ctx.has_acct_plugin = False

    release_version = "release-23.0"
    open_api_spec = f"https://raw.githubusercontent.com/kubernetes-client/python/{release_version}/scripts/swagger.json"
    resource = hub.OPT.pop_create.resource or "all"

    # Download and parse open-api spec for k8s python client to get plugin metadata
    plugins = {}
    with urllib.request.urlopen(open_api_spec) as f:
        k8s_openapi_spec_str = f.read().decode("utf-8")
        if k8s_openapi_spec_str:
            plugins = hub.pop_create.k8s.plugin.get_plugins(
                k8s_openapi_spec_str=k8s_openapi_spec_str, resource=resource
            )

    # Initialize cloud spec
    ctx.cloud_spec = NamespaceDict(
        api_version=release_version,
        project_name=ctx.project_name,
        service_name=ctx.service_name,
        request_format={
            "present": hub.pop_create.k8s.template.PRESENT_REQUEST_FORMAT,
            "absent": hub.pop_create.k8s.template.ABSENT_REQUEST_FORMAT,
            "describe": hub.pop_create.k8s.template.DESCRIBE_REQUEST_FORMAT,
        },
        plugins=plugins,
    )

    hub.cloudspec.init.run(
        ctx,
        directory,
        create_plugins=["state_modules"],
    )
    ctx.cloud_spec.plugins = {}
    return ctx



"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

rbac.authorization.k8s.io/v1/ClusterRoleBinding
"""
import copy
from typing import Any
from typing import Dict
from typing import List

from deepdiff import DeepDiff

__contracts__ = ["resource"]


async def present(
    hub,
    ctx,
    name: str,
    metadata: Dict,
    role_ref: Dict,
    subjects: List,
    resource_id: str = None,
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    create a ClusterRoleBinding

    Args:
        name(Text): An Idem name of the resource.
        resource_id(Text, optional): An identifier of the resource in the provider. Defaults to None.
        metadata(Dict): Standard object's metadata.
        role_ref(Dict): RoleRef can only reference a ClusterRole in the global namespace. If the RoleRef cannot be
            resolved, the Authorizer must return an error.
        subjects(List): Subjects holds references to the objects the role applies to.
    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              k8s.rbac.v1.cluster_role_binding.present:
                  - name: idem-test-cluster-role-binding
                  - metadata:
                      annotations:
                        rbac.authorization.kubernetes.io/autoupdate: 'true'
                      labels:
                        kubernetes.io/bootstrapping: rbac-defaults
                      name: idem-test-cluster-role-binding
                  - role_ref:
                      api_group: rbac.authorization.k8s.io
                      kind: ClusterRole
                      name: idem-test-cluster-role-binding
                  - subjects:
                    - api_group: rbac.authorization.k8s.io
                      kind: User
                      name: idem-test-cluster-role-binding
    """

    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)

    # Check for existing cluster_role_binding by name
    before = None
    if resource_id:
        cluster_role_binding = (
            await hub.exec.k8s.client.RbacAuthorizationV1Api.read_cluster_role_binding(
                ctx, name=resource_id
            )
        )
        if not cluster_role_binding["result"]:
            result["comment"] = cluster_role_binding["comment"]
            result["result"] = cluster_role_binding["result"]
            return result
        before = cluster_role_binding["ret"]

    # Update current state
    current_state = hub.tool.k8s.rbac.v1.cluster_role_binding_utils.convert_raw_cluster_role_binding_to_present(
        cluster_role_binding=before
    )
    result["old_state"] = current_state

    # Handle no change behaviour
    desired_state = {
        "resource_id": resource_id,
        "metadata": metadata,
        "role_ref": role_ref,
        "subjects": subjects,
    }

    is_change_detected = before is None or bool(
        DeepDiff(current_state, desired_state, ignore_order=True)
    )

    if not is_change_detected:
        result["comment"] = hub.tool.k8s.comment_utils.already_exists_comment(
            resource_type="k8s.rbac.v1.cluster_role_binding",
            name=name,
        )
        result["new_state"] = copy.deepcopy(result["old_state"])
        return result

    # Handle test behaviour
    if ctx.get("test", False):
        result["new_state"] = hub.tool.k8s.test_state_utils.generate_test_state(
            enforced_state=current_state,
            desired_state=desired_state,
        )
        result["comment"] = (
            hub.tool.k8s.comment_utils.would_update_comment(
                resource_type="k8s.rbac.v1.cluster_role_binding",
                name=name,
            )
            if before
            else hub.tool.k8s.comment_utils.would_create_comment(
                resource_type="k8s.rbac.v1.cluster_role_binding",
                name=name,
            )
        )
        return result

    # Handle actual resource create or update
    body = hub.tool.k8s.marshaller.unmarshal(
        desired_state=desired_state, k8s_model_name="V1ClusterRoleBinding"
    )
    if before:
        ret = await hub.exec.k8s.client.RbacAuthorizationV1Api.replace_cluster_role_binding(
            ctx, name=resource_id, body=body
        )
        result["result"] = ret["result"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = hub.tool.k8s.comment_utils.update_comment(
            resource_type="k8s.rbac.v1.cluster_role_binding",
            name=name,
        )
    else:
        ret = await hub.exec.k8s.client.RbacAuthorizationV1Api.create_cluster_role_binding(
            ctx, body=body
        )
        resource_id = body.metadata.name
        result["result"] = ret["result"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = hub.tool.k8s.comment_utils.create_comment(
            resource_type="k8s.rbac.v1.cluster_role_binding",
            name=name,
        )

    # Fetch the updated resource and update new_state
    cluster_role_binding = (
        await hub.exec.k8s.client.RbacAuthorizationV1Api.read_cluster_role_binding(
            ctx, name=resource_id
        )
    )
    if not cluster_role_binding["result"]:
        result["comment"] = result["comment"] + cluster_role_binding["comment"]
        result["result"] = cluster_role_binding["result"]
        return result
    after = cluster_role_binding["ret"]
    result[
        "new_state"
    ] = hub.tool.k8s.rbac.v1.cluster_role_binding_utils.convert_raw_cluster_role_binding_to_present(
        cluster_role_binding=after
    )
    return result


async def absent(hub, ctx, name: str, resource_id: str = None) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    delete a ClusterRoleBinding

    Args:
        name(Text): An Idem name of the resource.
        resource_id(Text, optional): An identifier of the resource in the provider. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              k8s.rbac.v1.cluster_role_binding.absent:
                - name: value
                - resource_id: value
    """

    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)

    before = None
    if resource_id:
        cluster_role_binding = (
            await hub.exec.k8s.client.RbacAuthorizationV1Api.read_cluster_role_binding(
                ctx, name=resource_id
            )
        )
        if cluster_role_binding and cluster_role_binding["result"]:
            before = cluster_role_binding["ret"]
            result[
                "old_state"
            ] = hub.tool.k8s.rbac.v1.cluster_role_binding_utils.convert_raw_cluster_role_binding_to_present(
                cluster_role_binding=before
            )

    if not before:
        result["comment"] = hub.tool.k8s.comment_utils.already_absent_comment(
            resource_type="k8s.rbac.v1.cluster_role_binding",
            name=name,
        )
    elif ctx.get("test", False):
        result["comment"] = hub.tool.k8s.comment_utils.would_delete_comment(
            resource_type="k8s.rbac.v1.cluster_role_binding",
            name=name,
        )
    else:
        ret = await hub.exec.k8s.client.RbacAuthorizationV1Api.delete_cluster_role_binding(
            ctx, name=resource_id
        )
        if not ret["result"]:
            result["result"] = ret["result"]
            result["comment"] = ret["comment"]
            return result

        result["comment"] = hub.tool.k8s.comment_utils.delete_comment(
            resource_type="k8s.rbac.v1.cluster_role_binding",
            name=name,
        )
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    list or watch objects of kind ClusterRoleBinding


    Returns:
        Dict[str, Dict[str, Any]]

    Examples:

        .. code-block:: bash

            $ idem describe k8s.rbac.v1.cluster_role_binding
    """

    ret = await hub.exec.k8s.client.RbacAuthorizationV1Api.list_cluster_role_binding(
        ctx,
    )
    if not ret["result"]:
        hub.log.debug(f"Could not describe cluster_role_binding {ret['comment']}")
        return {}

    result = {}
    for cluster_role_binding in ret["ret"].items:
        cluster_role_binding_resource = hub.tool.k8s.rbac.v1.cluster_role_binding_utils.convert_raw_cluster_role_binding_to_present(
            cluster_role_binding=cluster_role_binding
        )
        result[cluster_role_binding.metadata.name] = {
            "k8s.rbac.v1.cluster_role_binding.present": [
                {parameter_key: parameter_value}
                for parameter_key, parameter_value in cluster_role_binding_resource.items()
            ]
        }
    return result



"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__

v1/Namespace
"""
import copy
from typing import Any
from typing import Dict
from typing import Tuple

from deepdiff import DeepDiff

__contracts__ = ["resource"]


async def present(
    hub, ctx, name: str, metadata: Dict, resource_id: str = None
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    create a Namespace

    Args:
        name(Text): An Idem name of the resource.
        resource_id(Text, optional): An identifier of the resource in the provider. Defaults to None.
        metadata(Dict): Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-
            architecture/api-conventions.md#metadata.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              k8s.core.v1.namespace.present:
              - name: idem-test
              - metadata:
                  name: idem-test
                  labels:
                    example-label-name: example-label-value
    """

    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)

    # Check for existing namespace by name
    before = None
    if resource_id:
        namespace = await hub.exec.k8s.client.CoreV1Api.read_namespace(
            ctx, name=resource_id
        )
        if not namespace["result"]:
            result["comment"] = namespace["comment"]
            result["result"] = namespace["result"]
            return result
        before = namespace["ret"]

    # Update current state
    current_state = (
        hub.tool.k8s.core.v1.namespace_utils.convert_raw_namespace_to_present(
            namespace=before
        )
    )
    result["old_state"] = current_state

    # Handle no change behaviour
    desired_state = {
        "resource_id": resource_id,
        "metadata": metadata,
    }
    is_change_detected = before is None or bool(
        DeepDiff(current_state, desired_state, ignore_order=True)
    )

    if not is_change_detected:
        result["comment"] = hub.tool.k8s.comment_utils.already_exists_comment(
            resource_type="k8s.core.v1.namespace", name=name
        )
        result["new_state"] = copy.deepcopy(result["old_state"])
        return result

    # Handle test behaviour
    if ctx.get("test", False):
        result["new_state"] = hub.tool.k8s.test_state_utils.generate_test_state(
            enforced_state=current_state,
            desired_state=desired_state,
        )
        result["comment"] = (
            hub.tool.k8s.comment_utils.would_update_comment(
                resource_type="k8s.core.v1.namespace", name=name
            )
            if before
            else hub.tool.k8s.comment_utils.would_create_comment(
                resource_type="k8s.core.v1.namespace", name=name
            )
        )
        return result

    # Handle actual resource create or update
    body = hub.tool.k8s.marshaller.unmarshal(
        desired_state=desired_state, k8s_model_name="V1Namespace"
    )
    if before:
        ret = await hub.exec.k8s.client.CoreV1Api.replace_namespace(
            ctx, name=resource_id, body=body
        )
        result["result"] = ret["result"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = hub.tool.k8s.comment_utils.update_comment(
            resource_type="k8s.core.v1.namespace", name=name
        )
    else:
        ret = await hub.exec.k8s.client.CoreV1Api.create_namespace(ctx, body=body)
        resource_id = body.metadata.name
        result["result"] = ret["result"]
        if not result["result"]:
            result["comment"] = ret["comment"]
            return result
        result["comment"] = hub.tool.k8s.comment_utils.create_comment(
            resource_type="k8s.core.v1.namespace", name=name
        )

    # Fetch the updated resource and update new_state
    namespace = await hub.exec.k8s.client.CoreV1Api.read_namespace(
        ctx, name=resource_id
    )
    if not namespace["result"]:
        result["comment"] = result["comment"] + namespace["comment"]
        result["result"] = namespace["result"]
        return result
    after = namespace["ret"]
    result[
        "new_state"
    ] = hub.tool.k8s.core.v1.namespace_utils.convert_raw_namespace_to_present(
        namespace=after
    )
    return result


async def absent(
    hub, ctx, name: str, resource_id: str = None, timeout: Dict = None
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    delete a Namespace

    Args:
        name(Text): An Idem name of the resource.
        resource_id(Text, optional): An identifier of the resource in the provider. Defaults to None.
        timeout(Dict, optional): Timeout configuration for resource deletion.
            * delete (Dict) -- Timeout configuration for resource deletion
                * delay -- The amount of time in seconds to wait between attempts. Defaults to 15
                * max_attempts -- Customized timeout configuration containing delay and max attempts. Defaults to 40. Defaults to None.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              k8s.core.v1.namespace.absent:
                - name: value
                - resource_id: value
    """

    result = dict(comment=(), old_state=None, new_state=None, name=name, result=True)

    before = None
    if resource_id:
        namespace = await hub.exec.k8s.client.CoreV1Api.read_namespace(
            ctx, name=resource_id
        )
        if namespace and namespace["result"]:
            before = namespace["ret"]
            result[
                "old_state"
            ] = hub.tool.k8s.core.v1.namespace_utils.convert_raw_namespace_to_present(
                namespace=before
            )

    if not before:
        result["comment"] = hub.tool.k8s.comment_utils.already_absent_comment(
            resource_type="k8s.core.v1.namespace", name=name
        )
    elif ctx.get("test", False):
        result["comment"] = hub.tool.k8s.comment_utils.would_delete_comment(
            resource_type="k8s.core.v1.namespace", name=name
        )
    else:
        ret = await hub.exec.k8s.client.CoreV1Api.delete_namespace(
            ctx, name=resource_id
        )
        if not ret["result"]:
            result["result"] = ret["result"]
            result["comment"] = ret["comment"]
            return result

        # Custom waiter for delete
        waiter_config = hub.tool.k8s.waiter_utils.create_waiter_config(
            timeout_config=timeout.get("delete") if timeout else None,
        )

        def delete_acceptor(
            status: bool, data: Dict = None, comment: Tuple = None
        ) -> bool:
            if not status and '"reason":"NotFound"' in comment[0]:
                return True
            if not status:
                raise RuntimeError(
                    f"Error while waiting for delete namespace : {comment[0]}"
                )
            return False

        arguments = {"name": resource_id}
        result_arguments = {"condition": "status.condition"}

        namespace_waiter = hub.tool.k8s.waiter_utils.create_waiter(
            api_class_name="CoreV1Api",
            operation="read_namespace",
            arguments=arguments,
            result_arguments=result_arguments,
            acceptor_function=delete_acceptor,
        )
        try:
            await hub.tool.k8s.custom_waiter.wait(
                ctx,
                waiter=namespace_waiter,
                waiter_config=waiter_config,
                err_graceful=True,
            )
        except Exception as e:
            result["comment"] = result["comment"] + (str(e),)
            result["result"] = False
            return result

        result["comment"] = hub.tool.k8s.comment_utils.delete_comment(
            resource_type="k8s.core.v1.namespace", name=name
        )
    return result


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    list or watch objects of kind Namespace


    Returns:
        Dict[str, Dict[str, Any]]

    Examples:

        .. code-block:: bash

            $ idem describe k8s.core.v1.namespace
    """

    ret = await hub.exec.k8s.client.CoreV1Api.list_namespace(
        ctx,
    )
    if not ret["result"]:
        hub.log.debug(f"Could not describe namespace {ret['comment']}")
        return {}

    result = {}
    for namespace in ret["ret"].items:
        namespace_resource = (
            hub.tool.k8s.core.v1.namespace_utils.convert_raw_namespace_to_present(
                namespace=namespace
            )
        )
        result[namespace.metadata.name] = {
            "k8s.core.v1.namespace.present": [
                {parameter_key: parameter_value}
                for parameter_key, parameter_value in namespace_resource.items()
            ]
        }
    return result
